function [ hnew, varargout ] = sparse_minimum_1_norm( h, wpr, wpt, wsr, wst )
%[ hnew ] = sparse_minimum_1_norm( h, wpr, wpt, wsr, wst )
%
% to be explained
%
% input:
%   h       impulse response of Filter
%   wpr     passband range
%               [min_p_w1, max_p_w1;
%                min_p_w2, max_p_w2;
%                ...]
%   wpt     passband threshold
%           tolerance about the ideal
%               [w1_p_range_threshold;
%                w2_p_range_threshold;
%                ...]
%   wsr     stopband range
%               [min_s_w1, max_s_w1;
%                min_s_w2, max_s_w2;
%                ...]
%   wst     stopband threshold
%               [w1_s_treshold;
%                w2_s_treshold;
%                ...]
%
% output:
%   hnew                optimized Filter
%   delta   (optional)  Errorvector
%
%---------------------------------------------------------------
%
%   Author      Ludwig Mohr
%   Date        January 2012
%   Subject     Advanced Signalprocessing 1, Seminar - WS 2011/12
%

[b, M] = getb(h);

K = 10 * M;
W = ones(K,1);
wk = linspace(-pi,pi,K)';

Z = [];     %index set of coefficients constrained to zero a priori
for i = 1:M
    if b(i) == 0
        Z = [Z; i];
    end
end
N = 1:M+1;
N(Z) = [];  %index set of coefficients not constrained to zero

bN = b(N);  %zero coefficients stripped from b
M1 = length(bN);    %length of bN

%obtain frequency response of digital filter characterized by h
[hval, hfreq] = freqz(h);
%create initial vector for frequency response boundaries
bound = ones(length(hval),1)*[inf, -inf];   %linear upper and lower bounds

%create threshold curve for stop condition in linprog cycles
%band-stop region
N_ws = length(wst);
for k = 1:N_ws
    range = find(hfreq > wsr(k,1) & hfreq < wsr(k,2));
    bound(range, 1) = wst(k);
end

%pass band region
N_wp = length(wpt);
wpo = 0;    %pass band region offset: deviation from 0dB
for k = 1:N_wp;
    range = find(hfreq > wpr(k,1) & hfreq < wpr(k,2));
    bound(range,1) = wpo + wpt(k);  %upper bound
    bound(range,2) = wpo - wpt(k);  %lower bound
end

%vars for linprog
A = (bN' * cos((0:M)' * wk'))';
H = A .* exp(-j * wk * M);

Akn = W * ones(1,M + 1) .* cos(wk * (0:M));
h = W .* H;
e = ones(K,1);

AN = Akn(:,N);

%% stage 1 - obtain filter with small coefficients
%vars for stop condition
delta = .01;  %maximum deviation from original frequency response allowed

m = ones(M1, 1);
    
fl = [ones(M1, 1); zeros(M1, 1)];
Al = -([zeros(size(AN)), AN;
        zeros(size(AN)), -AN;
        eye(M1), eye(M1);
        eye(M1), -eye(M1);
        eye(M1), zeros(M1)]);
bl = -([abs(h) - delta * e;
        -abs(h) - delta * e;
        zeros(3*M1, 1)]);

[X,fval,exitflag] = linprog(fl, Al, bl);
disp(sprintf('Exitflag for optimization for small coefficients: %i', exitflag))

%get results from linear optimization
bN = X((length(bN) + 1):length(X));
m = X(1:length(m));

%new complete b vector (with zero coefficients from start)
b1 = b;
b1(N) = bN;
    
%get impulse response of new b vector
hnew = geth(b1);
[hval, hfreq] = freqz(hnew);

% %show frequency response for filter after stage 1 - for debugging purpose
% figure
% freqz(hnew)
% 
% resp = 20*log10(abs(hval));
% if (resp <= bound(:,1) & resp >= bound(:,2))
%     disp('still feasible after stage 1')
% else
%     disp('design not feasible after stage 1')
% end

%% stages 2 and 3 - create sparse filter via binary search of maximum number of zero-forced coefficients

%init value
delta2 = 1;
it = 0;

%sorted version of coefficients w/ index vector to determin the M - J
%smallest coefficients
[bSort(:,1), bSort(:,2)] = sort(abs(b1));
J = NaN;    %will hold smallest number of non-zero coeffs still yielding a feasible design

%init pointers for binary search
lBnd = 1;
uBnd = M + 1;
mid = floor((lBnd + uBnd) / 2);

%search algorithm:
%J is the number of largest coefficients allowed to be non zero in bN
%do a binary search in Jvec for smallest J which still yields a feasible
%filter
%start with half of the coefficients set to zero. if the filter is
%feasible, search in the subarray to the left (lesser number of
%coefficients remain non-zero), if the resulting filter is not feasible
%continue search in the right subarray (higher number of coefficients
%remain non-zero.
%continue until the lower boundary of the subarray exceeds the upper
%boundary.
%feasible designs encountered are stored in b2 (together with information
%on how many coefficients were allowed to remain non-zero. if in a later
%loop iteration a feasible design is encountered again, this
%design will have more coefficients set to zero.
%when the break condition ist reached b2 will have the latest feasible
%design (the one with a maximum number of coefficients set to zero)

b2 = b1;
while(it < M)
    it = it + 1;    %emergency fail safe to prevent infinite loop
    %check if lower bound exceeds, if so break
    if uBnd <= lBnd
        break
    end
    
    Z = bSort(1:(M + 1 - mid),2);     %indices of coefficients set to zero
    N = bSort((M + 1 - mid + 1):M + 1,2);  %indices of non-zero coefficients
    
    AN = Akn(:,N);
    bnew = b1;
    bnew(Z) = 0;
    
    fl = [delta2;zeros(length(N),1)];
    Al = -([[e, AN]; [e, -AN]]);
    bl = -([abs(h);-abs(h)]);

    [X,fval,exitflag] = linprog(fl, Al, bl);
    disp(sprintf('Exitflag for iteration %i, %i non-zero coefficients: %i', it, mid, exitflag))
    delta2 = X(1);
    deltaV(it) = delta2;
    bnew(N) = X(2:length(N) + 1);

    %calculate new impulse response
    htemp = geth(bnew);
    [hval, hfreq] = freqz(htemp);
    
    %calculate if new filter lies within acceptable boundaries
    resp = 20*log10(abs(hval));
    
    %check in which direction search will be continued
    %still feasible?
    if (resp <= bound(:,1) & resp >= bound(:,2))
        %continue with fewer non-zero coefficients
        uBnd = mid - 1;
        b2 = bnew;  %backup latest feasible design
        J = mid;    %how many coefficients remain non-zero in this design
        % disp('feasible')
    else
        %filter design not feasible, continue with more non-zero
        %coefficients
        lBnd = mid + 1;
        % disp('not feasible')
    end
    %new middle of search subarray
    mid = floor((lBnd + uBnd) / 2);
    
end

hnew = geth(b2);

disp(sprintf('Number of zeros in b: %i', (M - J + 1)))

% figure
% plot(deltaV)

varargout = {deltaV};
varargout(2) = {Z};

end
