function [ hnew, varargout ] = sparse_smallest_increase( h, wpr, wpt, wsr, wst )
%[ hnew ] = sparse_smallest_increase( h, wpr, wpt, wsr, wst )
%[ hnew, delta ] = sparse_smallest_increase( h, wpr, wpt, wsr, wst )
%[ hnew, delta, Z ] = sparse_smallest_increase( h, wpr, wpt, wsr, wst )
%
% Create Sparse impulse Response for a FIR Type 1 filter.
% Algorithm:
%   - Set each coefficient seperately to zero and calculate the delta
%   - Take the solution with the smallest increase of the error
% Repeat till frequency bounds are reached
%
% input:
%   h       impulse response of Filter
%   wpr     passband range
%               [min_p_w1, max_p_w1;
%                min_p_w2, max_p_w2;
%                ...]
%   wpt     passband threshold
%           tolerance about the ideal
%               [w1_p_range_threshold;
%                w2_p_range_threshold;
%                ...]
%   wsr     stopband range
%               [min_s_w1, max_s_w1;
%                min_s_w2, max_s_w2;
%                ...]
%   wst     stopband threshold
%               [w1_s_treshold;
%                w2_s_treshold;
%                ...]
%
% output:
%   hnew                optimized Filter
%   delta   (optional)  Errorvector
%   Z       (optional)  Vector with the Zero-Elements
%
%---------------------------------------------------------------
%
%   Author      Peter Reinprecht
%   Date        Januar 2012
%   Course      Advanced Signalprocessing 1, Seminar - WS 2011/12
%

[b, M] = getb(h);

[hval, hfreq] = freqz(h);
resp = 20*log10(abs(hval));

%% define frequency bounds
bound = ones(length(hval),1)*[inf, -inf];  %[upper bound, lower bound] LINEAR!!!

%stopband;
N_ws = length(wst);
for k = 1:N_ws
    range = find(hfreq > wsr(k,1) & hfreq < wsr(k,2));
    bound(range, 1) = wst(k);
end
stoprange = range;

%passband
N_wp = length(wpt);
wpassoffset = 0;
for k = 1:N_wp
    range = find(hfreq > wpr(k,1) & hfreq < wpr(k,2));
    bound(range, 1) = wpassoffset + wpt(k);   %upper bound
    bound(range, 2) = wpassoffset - wpt(k);   %lower bound
end

%% set up initial parameter
K = 10*M;
W = ones(K,1);
wk = linspace(-pi,pi,K)';

%generate variables
A = (b' * cos((0:M)'*wk'))'; %formula (2)
Hd = A.*exp(-1i*wk*M);        %frequency response of (1)

Akn = W*ones(1,M+1).*cos(wk*(0:M));
hk = W.*Hd;
ek = ones(K,1);

%init value
Al = [[-ek, Akn]; [-ek, -Akn]];
bl = [abs(hk);-abs(hk)];
beql = zeros(M+2,1);

delta = 0.1;
deltaV = [];    %vector with delta
N = 1:M+1;      %vector with index of all nonzero coefficients
Z = [];         %vector with index of all zero coefficents
it = 0;
error_dB = [];

%% optimize till bounds are reached
%while( upper bound & lower bound & number of iterations reasonable)
while(resp <= bound(:,1) & resp >= bound(:,2) & it<M)
    it = it +1;
    b_backup = b;   %backup old b
    Z_backup = Z;
    deltaV_backup = deltaV;
    error_dB_backup = error_dB;

    deltaV(it) = 1; %define max error level (compare with calculation)

    for index_it = N
        N_temp = N;
        N_temp(N_temp==index_it) = [];

        %calculate new input vectors
        AN = Akn(:,N_temp);

        fl = [delta;zeros(M+1-it,1)];
        Al = -([[ek, AN]; [ek, -AN]]);
        bl = -([abs(hk);-abs(hk)]);

        [X,fval,exitflag] = linprog(fl, Al, bl);
        disp(sprintf('Exitflag for iteration %i/%i: %i ', it, index_it, exitflag))

        %is the new value smaller then the others?
        if X(1) < deltaV(it)
            min_pos = index_it;
            deltaV(it, 1) = X(1);
            b(N_temp) = X(2:M+2-it);
            b(min_pos) = 0;
        end
    end
    
    Z(it) = min_pos;
    N(N==min_pos) = [];

    %calculate new impulse response
    htemp = geth(b);
    [hval, hfreq] = freqz(htemp);
    resp = 20*log10(abs(hval));
%     deltaV(it, 2) = max(20*log10(abs(hval(stoprange))));
    error_dB(it) = max(20*log10(abs(hval(stoprange))));
end

%% write output
if it>=M    %if numbers of iteration is not reasonable anymore
    b_backup = b;   %backup old b
    Z_backup = Z;
    delta_backup = deltaV;
    delta_backup = deltaV;
end
hnew = geth(b_backup);
varargout(1) = {deltaV_backup};
varargout(2) = {Z_backup};
varargout(3) = {error_dB_backup};

end

